#include "filesettingspanel.h"
#include <vector>

enum
{
	BTN_ADD = wxID_HIGHEST,
	BTN_CHANGE,
	BTN_DELETE,
	O_FILE_LIST,
	
};

BEGIN_EVENT_TABLE(FileSettingsPanel, wxPanel)
	EVT_BUTTON(BTN_ADD, FileSettingsPanel::on_btnAdd)
	EVT_BUTTON(BTN_CHANGE, FileSettingsPanel::on_btnChange)
	EVT_BUTTON(BTN_DELETE, FileSettingsPanel::on_btnDelete)
	EVT_LISTBOX(O_FILE_LIST, FileSettingsPanel::on_oFileList)
END_EVENT_TABLE()

FileSettingsPanel::FileSettingsPanel(wxWindow *oParent, Settings *oNewSettings)
: wxPanel(oParent, -1)
{
	// Sizer creation.
	oWindowSizer = new wxBoxSizer(wxVERTICAL);
	oMainSizer = new wxBoxSizer(wxHORIZONTAL);
	oExtensionSizer = new wxStaticBoxSizer(oExtensionBox, wxVERTICAL);
	oInputSizer = new wxFlexGridSizer(2, 3);
	oButtonSizer = new wxBoxSizer(wxHORIZONTAL);
	
	oInputSizer->AddGrowableCol(1);
	
	// Control creation.
	oFileList = new wxListBox(this, O_FILE_LIST);
	
	lblType = new wxStaticText(this, -1, _("Type: File"));
	lblTypeValue = new wxStaticText(this, -1, _(""));
	
	lblEnding = new wxStaticText(this, -1, _("Ending:"));
	lblRun = new wxStaticText(this, -1, _("Run:"));
	lblTerminal = new wxStaticText(this, -1, _(""));
	
	txtEnding = new wxTextCtrl(this, -1);
	txtRun = new wxTextCtrl(this, -1);
	btnTerminal = new wxCheckBox(this, -1, _("Show output"));
	
	btnAdd = new wxButton(this, BTN_ADD, _("Add"));
	btnChange = new wxButton(this, BTN_CHANGE, _("Change"));
	btnDelete = new wxButton(this, BTN_DELETE, _("Delete"));
	
	// wxStaticBox thingee.
	oExtensionBox = new wxStaticBox(this, -1, _("Extension settings"));
	oExtensionSizer = new wxStaticBoxSizer(oExtensionBox, wxVERTICAL);
	
	// Sizer manipulation.
	oButtonSizer->Add(btnAdd, 1, wxALL, 4);
	oButtonSizer->Add(btnChange, 1, wxALL, 4);
	oButtonSizer->Add(btnDelete, 1, wxALL, 4);
	
	oInputSizer->Add(lblType, 0, wxALL, 4);
	oInputSizer->Add(lblTypeValue, 1, wxGROW | wxTOP, 4);
	oInputSizer->Add(lblEnding, 0, wxALL, 4);
	oInputSizer->Add(txtEnding, 1, wxGROW);
	oInputSizer->Add(lblRun, 0, wxALL, 4);
	oInputSizer->Add(txtRun, 1, wxGROW);
	oInputSizer->Add(lblTerminal, 0, wxALL, 4);
	oInputSizer->Add(btnTerminal, 1, wxGROW, 4);
	
	oExtensionSizer->Add(oInputSizer, 0, wxGROW);
	oExtensionSizer->Add(oButtonSizer, 0, wxGROW);
	
	oMainSizer->Add(oFileList, 0, wxGROW);
	oMainSizer->Add(oExtensionSizer, 1, wxGROW | wxLEFT, 4);
	
	oWindowSizer->Add(oMainSizer, wxALL, 4);
	
	// Formalities.
	SetAutoLayout(true);
	SetSizer(oWindowSizer);
	oWindowSizer->Fit(this);
	
	// Finally...
	oSettings = oNewSettings;
	readSettings();
}

void FileSettingsPanel::on_btnAdd(wxCommandEvent &oEvent)
{
	if (oSettings->addFileEnding((char *)txtEnding->GetValue().c_str(), (char *)txtRun->GetValue().c_str(), (char *)btnTerminal->GetValue()))
	{
		readSettings();
		oFileList->SetStringSelection(txtEnding->GetValue());
		txtEnding->SetValue(_(""));
		txtRun->SetValue(_(""));
		btnTerminal->SetValue(false);
		txtEnding->SetFocus();
	}
}

void FileSettingsPanel::on_btnChange(wxCommandEvent &oEvent)
{
	if (oSettings->changeFileEnding((char *)oFileList->GetStringSelection().c_str(), (char *)txtEnding->GetValue().c_str(), (char *)txtRun->GetValue().c_str(), (char *)btnTerminal->GetValue()))
	{
		readSettings();
		oFileList->SetStringSelection(txtEnding->GetValue());
		txtEnding->SetValue(_(""));
		txtRun->SetValue(_(""));
		btnTerminal->SetValue(false);
		txtEnding->SetFocus();
	}
}

void FileSettingsPanel::on_btnDelete(wxCommandEvent &oEvent)
{
	if (oSettings->deleteFileEnding((char *)oFileList->GetStringSelection().c_str()))
	{
		readSettings();
		txtEnding->SetValue(oFileList->GetStringSelection());
		txtRun->SetValue((wxChar *)oSettings->getRunString((char *)oFileList->GetStringSelection().c_str()).c_str());
		txtEnding->SetFocus();
	}
}

void FileSettingsPanel::readSettings()
{
	std::vector<std::string> sEndings;
	sEndings = oSettings->getFileEndings();
	oFileList->Clear();
	for (unsigned int i = 0; i < sEndings.size(); i++)
	{
		oFileList->Append((wxChar *)sEndings[i].c_str());
	}
}

void FileSettingsPanel::loadSelection(const wxString &sExtension)
{
	
	txtEnding->SetValue(sExtension);
	txtRun->SetValue((wxChar *)oSettings->getRunString((char *)sExtension.c_str()).c_str());
	btnTerminal->SetValue((wxChar *)oSettings->getTerminal((char *)sExtension.c_str()));
	txtEnding->SetFocus();
}

void FileSettingsPanel::on_oFileList(wxCommandEvent &oEvent)
{
	loadSelection(oFileList->GetStringSelection());
}


