#include "filetree.h"
#include "droptarget.h"
#include "uthread.h"

enum
{
	O_TREE = wxID_HIGHEST,
};

BEGIN_EVENT_TABLE(UFiletree, wxTreeCtrl)
	EVT_TREE_BEGIN_DRAG(O_TREE, UFiletree::OnDragBegin)
	EVT_TREE_END_DRAG(O_TREE, UFiletree::OnDragEnd)
	EVT_TREE_ITEM_EXPANDING(O_TREE, UFiletree::OnItemExpanding)
	EVT_TREE_ITEM_COLLAPSED(O_TREE, UFiletree::OnItemCollapsed)
	EVT_TREE_ITEM_ACTIVATED(O_TREE, UFiletree::OnItemActivated)
	EVT_TREE_BEGIN_LABEL_EDIT(O_TREE, UFiletree::OnItemBeginRename)
	EVT_TREE_END_LABEL_EDIT(O_TREE, UFiletree::OnItemEndRename)
END_EVENT_TABLE()

UFiletree::UFiletree(wxWindow *oParent, wxWindowID iID, wxTextCtrl *oLocation, MainWindow *oParentWindow, Settings *oNewSettings)
: wxTreeCtrl(oParent, O_TREE, wxDefaultPosition, wxDefaultSize, wxTR_HAS_BUTTONS | wxTR_TWIST_BUTTONS | wxTR_DEFAULT_STYLE | wxTR_ROW_LINES | wxTR_MULTIPLE)
{
	bSeeHidden = false;
	oSettings = oNewSettings;
	txtLocation = oLocation;
	oParentFrame = oParentWindow;
	SetIndent(14);
	SetSpacing(8);
	SetBackgroundColour(wxColour(240, 240, 240));
	SetDropTarget(new UDropTarget());
	selectTreeItem(GetRootItem());
	
	createThread();
}

void UFiletree::createThread()
{
	oThread = new UThread(this);
	oThread->Create();
	oThread->Run();
}

wxString UFiletree::getFullpath(wxTreeItemId iItem)
{
	if (!itemExists(iItem))
	{
		return wxString(_("!"));
	}
	wxTreeItemId iNewID = iItem;
	wxString sDir;
	sDir = GetItemText(iItem);
	while (iNewID != GetRootItem())
	{
		iNewID = GetParent(iNewID);
		sDir = GetItemText(iNewID) + _("/") + sDir;
	}
	sDir.Replace(_("//"), _("/"));
	return sDir;
}

bool UFiletree::deleteItem()
{
	if (!somethingSelected())
	{
		return false;
	}
	
	wxTreeItemId iItemID = getTreeSelection();
	
	// Security measures.
	// Also, what are we going to view if we delete everything we see?
	if (GetRootItem() == iItemID)
	{
		return false;
	}
	
	wxMessageDialog oDialog(this, _("Are you sure you want to delete this item?"), _("Delete item"), wxYES_NO);
	int iResult = oDialog.ShowModal();
	if (iResult == wxID_YES)
	{
		wxString sItem = getFullpath(iItemID);
		wxString sRunString;
		if (wxDir::Exists(sItem))
		{
			sRunString = _("rm -rf \"");
			oTimeMonitor.delTime((char *)sItem.c_str());
		}
		else
		{
			sRunString = _("rm -f \"");
		}
		sRunString += sItem + _("\"");
		iResult = system((char *)sRunString.c_str());
		if (iResult == 0)
		{
			wxTreeItemId iParent;
			wxTreeItemId iNext;
			wxTreeItemId iPrev;
			iParent = GetParent(iItemID);
			wxString sParentDir;
			sParentDir = getFullpath(iParent);
			iNext = GetNextSibling(iItemID);
			iPrev = GetPrevSibling(iItemID);
			
			if (itemExists(iNext))
			{
				//selectTreeItem(iNext);
			}
			else if (itemExists(iPrev))
			{
				selectTreeItem(iPrev);
			}
			else
			{
				selectTreeItem(iParent);
			}
			
			wxDir oDir(sParentDir);
			if (!oDir.HasFiles() && !oDir.HasSubDirs())
			{
				Collapse(iParent);
				SetItemHasChildren(iParent, false);
			}
			return true;
		}
	}
	return false;
}

void UFiletree::selectPrevItem()
{
	selectTreeItem(getPrevID(getTreeSelection()));
}

void UFiletree::OnDragBegin(wxTreeEvent &oEvent)
{
	iMovingID = oEvent.GetItem();
	oEvent.Allow();
}

void UFiletree::OnDragEnd(wxTreeEvent &oEvent)
{
	wxString sOldItem;
	wxString sDestination;
	wxString sRunString;
	int iResult;
	if (wxDir::Exists(getFullpath(oEvent.GetItem())))
	{
		sOldItem = getFullpath(iMovingID);
		sDestination = getFullpath(oEvent.GetItem());
		if (sOldItem == sDestination + _("/") + GetItemText(iMovingID))
		{
			return;
		}
		sRunString = _("mv \"") + sOldItem + _("\" \"") + sDestination + _("\"");
		iResult = system((char *)sRunString.c_str());
	}
}

wxTreeItemId UFiletree::addItem(wxTreeItemId iDirID, const wxString &sItem)
{
	wxTreeItemId iTempID;
	wxString sFullpath(getFullpath(iDirID) + _("/") + sItem);
	int iImage;
	if (wxDir::Exists(sFullpath))
	{
		if (wxFile::Access(sFullpath, wxFile::read))
		{
			iTempID = AppendItem(iDirID, sItem, (UFileHandler::isSymlink((char *)sFullpath.c_str()) ? 6 : 1));
			wxDir oDir(sFullpath);
			if (oDir.HasFiles() || oDir.HasSubDirs())
			{
				SetItemHasChildren(iTempID, true);
			}
		}
		else
		{
			iTempID = AppendItem(iDirID, sItem, 3);
		}
	}
	else
	{
		if (UFileHandler::isSymlink((char *)sFullpath.c_str()))
		{
			iImage = (UFileHandler::isExecutable((char *)sFullpath.c_str()) ? 5 : 4);
		}
		else
		{
			iImage = (UFileHandler::isExecutable((char *)sFullpath.c_str()) ? 2 : 0);
		}
		
		iTempID = AppendItem(iDirID, sItem, iImage);
	}
	SetItemTextColour(iTempID, wxColour(70, 70, 70));
	return iTempID;
}

void UFiletree::OnItemExpanding(wxTreeEvent &oEvent)
{
	wxString sDir(getFullpath(oEvent.GetItem()));
	
	ReloadDir(oEvent.GetItem(), sDir, false);
}

void UFiletree::OnItemCollapsed(wxTreeEvent &oEvent)
{
	wxString sFullpath(getFullpath(oEvent.GetItem()));
	wxDir oDir(sFullpath);

	DeleteChildren(oEvent.GetItem());
	if (oDir.HasFiles() || oDir.HasSubDirs())
	{
		SetItemHasChildren(oEvent.GetItem(), true);
	}
}

void UFiletree::OnItemActivated(wxTreeEvent &oEvent)
{
	Activate(oEvent.GetItem());
}

void UFiletree::OnItemBeginRename(wxTreeEvent &oEvent)
{
	sTempItemLabel = GetItemText(oEvent.GetItem());
}

void UFiletree::OnItemEndRename(wxTreeEvent &oEvent)
{
	if (!Rename(oEvent.GetItem(), oEvent.GetLabel()))
	{
		SetItemText(oEvent.GetItem(), sTempItemLabel);
	}
}

void UFiletree::ReloadDir(wxTreeItemId iItemID, const wxString &sOptDir, bool bCheckOnly)
{
	/* I don't know if we need this.
	 * I was trying to reproduce a bug and this indicate it...
	 * but it sounds like a good idea anyway, so I just commented it and had it for keeps.
	if (!UFileHandler::isDir(sOptDir.c_str()))
	{
		return;
	}
	*/

	wxString sDir(sOptDir != _("") ? sOptDir : getFullpath(iItemID));
	wxDir oDir(sDir);
	wxString sItem;
	bool bCont = oDir.GetFirst(&sItem);
	if (GetChildrenCount(iItemID) == 0 && bCont)
	{
		SetItemHasChildren(iItemID, true);
		if (bCheckOnly)
		{
			return;
		}
	}
	while (bCont)
	{
		if ((sItem.Left(1) != _(".") || bSeeHidden) && !itemExists(sDir + _("/") + sItem))
		{
			addItem(iItemID, sItem);
		}
		bCont = oDir.GetNext(&sItem);
	}
	
	sItem = _("");
	std::vector<wxTreeItemId> iDeletedItems;
	wxTreeItemId iCurrent = GetRootItem();
	while (iCurrent = getNextID(iCurrent))
	{
		sItem = getFullpath(iCurrent);
		if (!wxDir::Exists(sItem) && !wxFile::Exists(sItem))
		{
			iDeletedItems.push_back(iCurrent);
		}
	}
	for (unsigned int i = 0; i < iDeletedItems.size(); i++)
	{
		if (getTreeSelection() == iDeletedItems[i])
		{
			wxTreeItemId iParent = GetParent(iDeletedItems[i]);
			wxString sParentDir(getFullpath(iParent));
			wxTreeItemId iNext = GetNextSibling(iDeletedItems[i]);
			wxTreeItemId iPrev = GetPrevSibling(iDeletedItems[i]);
			
			if (itemExists(iNext))
			{
				selectTreeItem(iNext);
			}
			else if (itemExists(iPrev))
			{
				selectTreeItem(iPrev);
			}
			else
			{
				selectTreeItem(iParent);
			}
			
			wxDir oDir(sParentDir);
			if (!oDir.HasFiles() && !oDir.HasSubDirs())
			{
				Collapse(iParent);
				SetItemHasChildren(iParent, false);
			}
		}
		if (itemExists(iDeletedItems[i]))
		{
			Delete(iDeletedItems[i]);
		}
	}
	
	SortChildren(iItemID);
}

void UFiletree::SetRoot(const wxString &sDir, bool bSkipLocationText)
{
	if (GetRootItem())
	{
		if (sDir == GetItemText(GetRootItem()) + _("/") || sDir + _("/") == GetItemText(GetRootItem()))
		{
			return;
		}
	}
	
	DeleteAllItems();
	if (!wxFile::Access(sDir, wxFile::read))
	{
		AddRoot(sDir, 3);
	}
	else
	{
		AddRoot(sDir, 1);
		SetItemTextColour(GetRootItem(), wxColour(70, 70, 70));
		SetItemHasChildren(GetRootItem(), true);
		Expand(GetRootItem());
	}
	
	// We do this only to avoid a neverending while-loop,
	// because when we change the value of txtLocation,
	// OnLocationChange() is called, which calls SetRoot
	// again. Which is bad.
	if (!bSkipLocationText)
	{
		txtLocation->SetValue(sDir);
	}
	
	oParentFrame->SetTitle(sDir);
	wxSetWorkingDirectory(sDir);
}

void UFiletree::Activate(wxTreeItemId iItemID)
{
	wxString sItem = getFullpath(iItemID);
	Activate(sItem);
}

void UFiletree::Activate(wxString sItem)
{
	if (wxDir::Exists(sItem))
	{
		sItem.Replace(_("//"), _("/"));
		SetRoot(sItem, false);
	}
	else
	{
		if (UFileHandler::isExecutable((char *)sItem.c_str()))
		{
			wxString sRunString;
			sRunString = sItem + _("&");
			system((char *)sRunString.c_str());
		}
		else
		{
			wxString sEnding;
			wxString sApp;
			sEnding = (wxChar *)oSettings->getEnding((char *)sItem.c_str()).c_str();
			if (oSettings->exists((char *)sEnding.c_str()))
			{
				sApp = (wxChar *)oSettings->getRunString((char *)sEnding.c_str()).c_str();
			}
			else
			{
				sApp = (wxChar *)oSettings->getGeneral("DEFAULT_APP").c_str();
			}
			if (sApp != _(""))
			{
				wxString sRunString;
				sRunString = wxString::Format(_("%s \"%s\""), sApp.c_str(), sItem.c_str());
				if (oSettings->getTerminal((char *)sEnding.c_str()))
				{
					run(sRunString, true);
				}
				else
				{
					system((char *)wxString(sRunString + _(" &")).c_str());
				}
			}
		}
	}
}

bool UFiletree::addFile()
{
	if (!somethingSelected())
	{
		return false;
	}
	
	wxTreeItemId iItemID = getTreeSelection();
	wxString sFilename;
	wxString sRunString;
	wxString sParentDir;
	wxString sAllegedDir;
	wxTreeItemId iParentID;
	int iResult;
	sFilename = wxGetTextFromUser(_("Enter name of new file"), _("Name"), _(""));
	
	if (sFilename == _(""))
	{
		return false;
	}
	
	sAllegedDir = getFullpath(iItemID);
	if (wxDir::Exists(sAllegedDir))
	{
		iParentID = iItemID;
		sParentDir = sAllegedDir;
	}
	else
	{
		iParentID = GetParent(iItemID);
		sParentDir = getFullpath(iParentID);
	}
	if (!wxFile::Exists(sParentDir + _("/") + sFilename))
	{
		sRunString = _("touch \"") + sParentDir + _("/") + sFilename + _("\"");
		iResult = system((char *)sRunString.c_str());
		if (iResult == 0)
		{
			return true;
		}
	}
	return false;
}

bool UFiletree::addFolder()
{
	if (!somethingSelected())
	{
		return false;
	}
	
	wxTreeItemId iItemID = getTreeSelection();
	wxString sFoldername;
	wxString sRunString;
	wxString sParentDir;
	wxString sAllegedDir;
	wxTreeItemId iParentID;
	int iResult;
	sFoldername = wxGetTextFromUser(_("Enter name of new folder"), _("Name"), _(""));
	
	if (sFoldername == _(""))
	{
		return false;
	}
	
	sAllegedDir = getFullpath(iItemID);
	if (wxDir::Exists(sAllegedDir))
	{
		iParentID = iItemID;
		sParentDir = sAllegedDir;
	}
	else
	{
		iParentID = GetParent(iItemID);
		sParentDir = getFullpath(iParentID);
	}
	if (!wxDir::Exists(sParentDir + _("/") + sFoldername))
	{
		sRunString = _("mkdir \"") + sParentDir + _("/") + sFoldername + _("\"");
		iResult = system((char *)sRunString.c_str());
		if (iResult == 0)
		{
			return true;
		}
	}
	return false;
}

bool UFiletree::Rename(wxTreeItemId iItemID, const wxString &sNewName)
{
	wxString sRunString;
	wxString sItem;
	wxString sNewItem;
	int iResult = 0;
	sItem = getFullpath(iItemID);
	sNewItem = sItem.Left(sItem.Length() - GetItemText(iItemID).Length()) + sNewName;
	if (sNewName.Length() > 0)
	{
		sRunString = _("mv \"") + sItem + _("\" \"") + sNewItem + _("\"");
		iResult = system((char *)sRunString.c_str());
	}
	return (iResult == 0);
}

void UFiletree::GoUp()
{
	wxString sOldPath = txtLocation->GetValue();
	wxString sNewPath;
	
	if (sOldPath.Right(1) == _("/"))
	{
		sOldPath = sOldPath.Left(sOldPath.Length() - 1);
	}
	
	sNewPath = sOldPath.Left(sOldPath.Find('/', true));
	
	if (sNewPath == _(""))
	{
		sNewPath = _("/");
	}

	if (sNewPath == sOldPath + _("/"))
	{
		return;
	}
	
	SetRoot(sNewPath, false);
	SetFocus();
}

void UFiletree::rightPress()
{
	long int iDummy = 0;
	wxString sFullpath;
	wxTreeItemId iItem;
	if (!somethingSelected())
	{
		return;
	}
	iItem = getTreeSelection();
	sFullpath = getFullpath(iItem);
	if (!wxDir::Exists(sFullpath))
	{
		return;
	}
	wxDir oDir(sFullpath);
	if (oDir.HasFiles() || oDir.HasSubDirs())
	{
		if (IsExpanded(iItem))
		{
			selectTreeItem(GetFirstChild(iItem, iDummy));
		}
		else
		{
			Expand(iItem);
		}
	}
}

void UFiletree::leftPress()
{
	if (!somethingSelected())
	{
		return;
	}
	
	if (IsExpanded(getTreeSelection()))
	{
		CollapseAndReset(getTreeSelection());
	}
	else if (GetParent(getTreeSelection()))
	{
		selectTreeItem(GetParent(getTreeSelection()));
	}
}

void UFiletree::selectNextItem()
{
	wxTreeItemId iItem;
	if (!somethingSelected())
	{
		return;
	}
	iItem = getTreeSelection();
	if (getNextID(iItem))
	{
		selectTreeItem(getNextID(getTreeSelection()));
	}
}

/*
void UFiletree::checkChanges()
{
	// Make sure something is selected.
	if (!this->somethingSelected())
	{
		this->selectTreeItem(this->GetRootItem());
	}
	
	wxTreeItemId iRoot = this->GetRootItem();
	wxTreeItemId iSelected;
	wxString sDir;
	std::vector<wxTreeItemId> iReload;
	std::vector<wxString> sReloadDirs;
	sDir = this->getFullpath(iRoot);
	if (this->oTimeMonitor.checkTime((char *)sDir.c_str()))
	{
		iReload.insert(iReload.begin(), iRoot);
		sReloadDirs.insert(sReloadDirs.begin(), sDir);
	}
	wxTreeItemId iCurrent = iRoot;
	while (iCurrent = this->getNextID(iCurrent))
	{
		sDir = this->getFullpath(iCurrent);
		if (this->oTimeMonitor.checkTime((char *)sDir.c_str()))
		{
			iReload.insert(iReload.begin(), iCurrent);
			sReloadDirs.insert(sReloadDirs.begin(), sDir);
		}
	}
	if (iReload.size() > 0)
	{
		for (unsigned int i = 0; i < iReload.size(); i++)
		{
			this->ReloadDir(iReload[i], sReloadDirs[i], true);
		}
	}
}
*/

void UFiletree::selectTreeItem(wxTreeItemId iItem)
{
	SelectItem(iItem);
}

void UFiletree::pressHome()
{
	selectTreeItem(GetRootItem());
}

void UFiletree::pressEnd()
{
	selectTreeItem(GetLastChild(GetRootItem()));
}

void UFiletree::openWith()
{
	wxTreeItemId iItem;
	if (iItem = getTreeSelection())
	{
		wxString sRunString;
		wxString sFullpath;
		sRunString = wxGetTextFromUser(_("Run with..."), _("Runstring:"), _("gvim '%s'"));
		if (sRunString.Find(_("%s")) == -1)
		{
			sRunString += _(" '%s'");
		}
		
		sFullpath = getFullpath(iItem);
		sRunString.Replace(_("%s"), sFullpath);
		wxExecute(sRunString);
	}
}

// Quite possibly the ugliest hack I've ever done.
// We iterate through the entire fucking tree to find out if something's selected.
// We do this because wxTreeCtrl is REALLY not working in regards to .IsOk().
// We'll get rid of this when we finally get around making sure that there's ALWAYS something selected.
bool UFiletree::somethingSelected()
{
	wxArrayTreeItemIds iSelections;
	return (GetSelections(iSelections) > 0);
}

void UFiletree::edit()
{
	if (!somethingSelected())
	{
		return;
	}
	EditLabel(getTreeSelection());
}

bool UFiletree::itemExists(const wxTreeItemId &iItem)
{
	wxTreeItemId iCurrent = GetRootItem();

	if (iCurrent == iItem)
	{
		return true;
	}

	bool bReturn = false;
	while (iCurrent = getNextID(iCurrent))
	{
		if (iCurrent == iItem)
		{
			bReturn = true;
			break;
		}
	}
	return bReturn;
	
}

bool UFiletree::itemExists(const wxString &sItem)
{
	wxTreeItemId iCurrent = GetRootItem();

	if (GetItemText(iCurrent) == sItem)
	{
		return true;
	}

	bool bReturn = false;
	while (iCurrent = getNextID(iCurrent))
	{
		if (getFullpath(iCurrent) == sItem)
		{
			bReturn = true;
			break;
		}
	}
	return bReturn;
}

// We'll do this until we get a real GetNextItem() function in wxWindows,
// one that simply, and only, fucking, gets the next item in the list, regardless of parenthood.
wxTreeItemId UFiletree::getNextID(const wxTreeItemId &iItem)
{
	wxTreeItemId iNext;
	wxTreeItemId iCurrent;
	if (IsExpanded(iItem) && GetChildrenCount(iItem) > 0)
	{
		long iDummy;
		return GetFirstChild(iItem, iDummy);
	}
	
	iCurrent = iItem;
	
	if (iNext = GetNextSibling(iCurrent))
	{
		return iNext;
	}
	
	while (GetItemParent(iCurrent))
	{
		iCurrent = GetItemParent(iCurrent);
		if (iNext = GetNextSibling(iCurrent))
		{
			return iNext;
		}
	}
	
	// And finally... we bail out like chickenshit, by knowingly returning something that isn't valid.
	// Although actually... it should never really come to this.
	// The while-loop above should always return iNext at some point.
	return GetNextSibling(iItem);
}

wxTreeItemId UFiletree::getPrevID(const wxTreeItemId &iItem)
{
	wxTreeItemId iCurrent = GetRootItem();
	
	if (iItem == iCurrent)
	{
		return iItem;
	}
	
	while (iItem != getNextID(iCurrent))
	{
		iCurrent = getNextID(iCurrent);
	}
	return iCurrent;
}

wxTreeItemId UFiletree::getTreeSelection()
{
	wxArrayTreeItemIds iSelections;
	GetSelections(iSelections);
	if (iSelections.GetCount() > 0)
	{
		return iSelections[0];
	}
	wxTreeItemId iSpareItem;
	return iSpareItem;
}

void UFiletree::run(const wxString &sCommand, bool bTerminal)
{
	oParentFrame->oProcessBox->setCommand(sCommand);
	
	if (!oParentFrame->oProcessBox->IsShown())
	{
		oParentFrame->oProcessBox->Show(true);
		oParentFrame->btnProcessClose->Show(true);
		oParentFrame->oMainSizer->Add(oParentFrame->oProcessBox, 0, wxGROW);
		oParentFrame->oMainSizer->Add(oParentFrame->btnProcessClose, 0, wxALIGN_CENTER, wxALIGN_CENTER);
		oParentFrame->oMainSizer->SetItemMinSize(oParentFrame->oProcessBox, 10, 120);
		oParentFrame->oMainSizer->Layout();
		oParentFrame->btnProcessClose->SetFocus();
	}
	
	oParentFrame->oProcessBox->run();
}

