#include <wx/image.h>

#include "mainwindow.h"

#include "images/file.xpm"
#include "images/app.xpm"
#include "images/app_symlink.xpm"
#include "images/file_symlink.xpm"
#include "images/folder.xpm"
#include "images/folder_pd.xpm"
#include "images/folder_symlink.xpm"
#include "images/up.xpm"
#include "images/home.xpm"
#include "images/fit.xpm"

enum
{
	O_TREE = wxID_HIGHEST,
	O_TIMER,
	TXT_LOCATION,
	TXT_RUN,
	MNU_NEWFOLDER,
	MNU_NEWFILE,
	MNU_OPENANDVIEW,
	MNU_OPENINNEW,
	MNU_OPENWITH,
	MNU_DELETE,
	MNU_EXIT,
	MNU_RENAME,
	MNU_HIDDEN,
	MNU_SHOW_MENUBAR,
	MNU_SHOW_LOCATION,
	MNU_SHOW_TOOLBAR,
	MNU_AUTO_FIT_TREE,
	MNU_UP,
	MNU_HOME,
	MNU_TREEFIT,
	MNU_SETTINGS,
	BTN_PROCESS_CLOSE,
	BTN_UP,
	BTN_HOME,
	BTN_FIT,
};

BEGIN_EVENT_TABLE(MainWindow, wxFrame)
	EVT_TIMER(O_TIMER, MainWindow::OnTimer)
	EVT_TEXT(TXT_LOCATION, MainWindow::OnLocationChange)
	EVT_TEXT_ENTER(TXT_LOCATION, MainWindow::OnLocationEnter)
	EVT_MENU(MNU_UP, MainWindow::OnUp)
	EVT_MENU(MNU_HOME, MainWindow::OnHome)
	EVT_MENU(MNU_NEWFOLDER, MainWindow::OnMenuNewFolder)
	EVT_MENU(MNU_NEWFILE, MainWindow::OnMenuNewFile)
	EVT_MENU(MNU_OPENANDVIEW, MainWindow::OnMenuOpenAndView)
	EVT_MENU(MNU_OPENINNEW, MainWindow::OnMenuOpenInNew)
	EVT_MENU(MNU_OPENWITH, MainWindow::OnMenuOpenWith)
	EVT_MENU(MNU_EXIT, MainWindow::OnMenuExit)
	EVT_MENU(MNU_RENAME, MainWindow::OnMenuRename)
	EVT_MENU(MNU_DELETE, MainWindow::OnMenuDelete)
	EVT_MENU(MNU_SHOW_MENUBAR, MainWindow::OnMenuShowMenubar)
	EVT_MENU(MNU_SHOW_LOCATION, MainWindow::OnMenuShowLocation)
	EVT_MENU(MNU_SHOW_TOOLBAR, MainWindow::OnMenuShowToolbar)
	EVT_MENU(MNU_AUTO_FIT_TREE, MainWindow::OnMenuAutoFitTree)
	EVT_MENU(MNU_HIDDEN, MainWindow::OnMenuHidden)
	EVT_MENU(MNU_SETTINGS, MainWindow::OnMenuSettings)
	EVT_TREE_KEY_DOWN(O_TREE, MainWindow::OnTreeKeyDown)
	EVT_TREE_ITEM_RIGHT_CLICK(O_TREE, MainWindow::OnRightClick)
	EVT_BUTTON(BTN_PROCESS_CLOSE, MainWindow::OnButtonProcessClose)
	EVT_BUTTON(BTN_UP, MainWindow::OnUp)
	EVT_BUTTON(BTN_HOME, MainWindow::OnHome)
	EVT_BUTTON(BTN_FIT, MainWindow::OnFit)
END_EVENT_TABLE()

MainWindow::MainWindow(const wxString &sTitle)
: wxFrame((wxFrame *)NULL, -1, sTitle, wxPoint(0, 0), wxSize(300, 400), wxDEFAULT_FRAME_STYLE, _("jaffm"))
{
	bLocationVisible = false;
	bToolbarVisible = false;
	bAutoFitTree = false;
	
	wxString sBaseDir(wxGetCwd());
	long iInterval = 400;
	
	oSettings = new Settings();
	
	oPanel = new wxPanel(this, -1);
	
	if (wxTheApp->argc > 1)
	{
		if (wxDir::Exists(wxTheApp->argv[1]) && wxString(wxTheApp->argv[1]) != _("."))
		{
			sBaseDir = wxTheApp->argv[1];
		}
		if (wxTheApp->argc > 2)
		{
			wxString(wxTheApp->argv[2]).ToLong(&iInterval);
		}
	}
	
	oMainSizer = new wxBoxSizer(wxVERTICAL);

	// Toolbar.
	oToolbarSizer = new wxBoxSizer(wxHORIZONTAL);
	wxBitmap imgUp  = (const char **)up_xpm;
	wxBitmap imgHome = (const char **)home_xpm;
	wxBitmap imgFit = (const char **)fit_xpm;
	btnUp = new wxBitmapButton(oPanel, BTN_UP, imgUp);
	btnHome = new wxBitmapButton(oPanel, BTN_HOME, imgHome);
	btnFit = new wxBitmapButton(oPanel, BTN_FIT, imgFit);
	btnUp->Show(false);
	btnHome->Show(false);
	btnFit->Show(false);

	// Resizing toolbar buttons to get rid of most of their borderes.
	wxSize oTempSize;
	oTempSize = wxSize(btnUp->GetSize().GetWidth() - 6, btnUp->GetSize().GetHeight() - 6);
	btnUp->SetSize(oTempSize);
	btnHome->SetSize(oTempSize);
	btnFit->SetSize(oTempSize);
	
	// Assigning toolbar buttons to toolbar.
	oToolbarSizer->Add(btnUp, 0, wxALL, 2);
	oToolbarSizer->Add(btnHome, 0, wxALL, 2);
	oToolbarSizer->Add(btnFit, 0, wxALL, 2);
	
	txtLocation = new wxTextCtrl(oPanel, TXT_LOCATION, sBaseDir, wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
	txtLocation->Show(false);
	
	oProcessBox = new UProcessBox(oPanel);
	oProcessBox->Show(false);
	
	btnProcessClose = new wxButton(oPanel, BTN_PROCESS_CLOSE, _("Close"));
	btnProcessClose->Show(false);
	
	wxDir oDir(wxGetCwd());
	
	wxString sFilename;
	
	oTree = new UFiletree(oPanel, O_TREE, txtLocation, this, oSettings);
	
	oImageList = new wxImageList(16, 16);
	
	wxBitmap imgFile;
	wxBitmap imgFolder;
	wxBitmap imgApp;
	wxBitmap imgFolder_PD;
	wxBitmap imgFile_Symlink;
	wxBitmap imgApp_Symlink;
	wxBitmap imgFolder_Symlink;
	
	imgFile = (const char **)file_xpm;
	imgFolder = (const char **)folder_xpm;
	imgApp = (const char **)app_xpm;
	imgFolder_PD = (const char **)folder_pd_xpm;
	imgFile_Symlink = (const char **)file_symlink_xpm;
	imgApp_Symlink = (const char **)app_symlink_xpm;
	imgFolder_Symlink = (const char **)folder_symlink_xpm;
	
	oImageList->Add(imgFile);
	oImageList->Add(imgFolder);
	oImageList->Add(imgApp);
	oImageList->Add(imgFolder_PD);
	oImageList->Add(imgFile_Symlink);
	oImageList->Add(imgApp_Symlink);
	oImageList->Add(imgFolder_Symlink);
	
	oTree->SetImageList(oImageList);
	
	wxIcon oIcon;
	oIcon.CopyFromBitmap(imgFolder);
	SetIcon(oIcon);
	
	oActionsMenu = new wxMenu();
	oActionsMenu->Append(MNU_NEWFOLDER, _("New Folder (&D)"));
	oActionsMenu->Append(MNU_NEWFILE, _("New File (&F)"));
	oActionsMenu->AppendSeparator();
	oActionsMenu->Append(MNU_OPENANDVIEW, _("Open And View Output"));
	oActionsMenu->Append(MNU_OPENINNEW, _("Open In New Window (&W)"));
	oActionsMenu->Append(MNU_OPENWITH, _("Open With... (&O)"));
	oActionsMenu->AppendSeparator();
	oActionsMenu->Append(MNU_RENAME, _("Rename (&R)"));
	oActionsMenu->Append(MNU_DELETE, _("Delete (D&el)"));
	oActionsMenu->AppendSeparator();
	oActionsMenu->Append(MNU_EXIT, _("E&xit (Esc)"));
	
	oMainSizer->Add(oTree, 1, wxGROW | wxALL, 2);
	
	oNavMenu = new wxMenu();
	
	mnuShowHidden = new wxMenuItem(oNavMenu, MNU_HIDDEN, _("Show Hidden (&H)"), wxEmptyString, true);
	mnuShowMenubar = new wxMenuItem(oNavMenu, MNU_SHOW_MENUBAR, _("Show Menubar (&M)"), wxEmptyString, true);
	mnuShowLocation = new wxMenuItem(oNavMenu, MNU_SHOW_LOCATION, _("Show Location (&L)"), wxEmptyString, true);
	mnuShowToolbar = new wxMenuItem(oNavMenu, MNU_SHOW_TOOLBAR, _("Show Toolbar (&T)"), wxEmptyString, true);
	mnuAutoFitTree = new wxMenuItem(oNavMenu, MNU_AUTO_FIT_TREE, _("Auto fit tree"), wxEmptyString, true);
	
	oNavMenu->Append(MNU_UP, _("Go Up (&Backspace)"));
	oNavMenu->Append(MNU_HOME, _("Go Home (&U)"));
	oNavMenu->AppendSeparator();
	oNavMenu->Append(mnuShowHidden);
	oNavMenu->Append(mnuShowMenubar);
	oNavMenu->Append(mnuShowLocation);
	oNavMenu->Append(mnuShowToolbar);
	oNavMenu->Append(mnuAutoFitTree);
	oNavMenu->AppendSeparator();
	oNavMenu->Append(MNU_SETTINGS, _("&Settings"));
	
	oMenuBar = new wxMenuBar();
	oMenuBar->Append(oActionsMenu, _("&Actions"));
	oMenuBar->Append(oNavMenu, _("&Navigation"));
	
	oPanel->SetSizer(oMainSizer);
	oPanel->SetAutoLayout(true);
	
	oTree->SetRoot(sBaseDir, true);
	
	if (oSettings->getGeneral("SHOW_MENUBAR") == "1")
	{
		toggleMenubar();
	}
	if (oSettings->getGeneral("SHOW_LOCATION") == "1")
	{
		toggleLocationBar();
	}
	if (oSettings->getGeneral("SHOW_TOOLBAR") == "1")
	{
		toggleToolbar();
	}
	if (oSettings->getGeneral("AUTO_FIT_TREE") == "1")
	{
		toggleAutoFit();
	}
	
	oSettingsWindow = new SettingsWindow(this, _("Settings"), oSettings);
	
	int iDummy, iWindowHeight, iTreeHeight;
	GetSize(&iDummy, &iWindowHeight);
	oTree->GetSize(&iDummy, &iTreeHeight);
	iRestHeight = iWindowHeight - iTreeHeight;
	
	oTimer = new wxTimer(this, O_TIMER);
	oTimer->Start(iInterval);
}

void MainWindow::OnLocationChange(wxCommandEvent &oEvent)
{
	if (wxDir::Exists(txtLocation->GetValue()))
	{
		oTree->SetRoot(txtLocation->GetValue(), true);
	}
}

void MainWindow::OnUp(wxCommandEvent &oEvent)
{
	oTree->GoUp();
}

void MainWindow::OnMenuRename(wxCommandEvent &oEvent)
{
	oTree->edit();
}

void MainWindow::OnMenuDelete(wxCommandEvent &oEvent)
{
	oTree->deleteItem();
}

void MainWindow::OnMenuNewFolder(wxCommandEvent &oEvent)
{
	oTree->addFolder();
}

void MainWindow::OnMenuNewFile(wxCommandEvent &oEvent)
{
	oTree->addFile();
}

void MainWindow::OnMenuShowMenubar(wxCommandEvent &oEvent)
{
	toggleMenubar();
}

void MainWindow::OnMenuShowLocation(wxCommandEvent &oEvent)
{
	toggleLocationBar();
}

void MainWindow::OnMenuShowToolbar(wxCommandEvent &oEvent)
{
	toggleToolbar();
}

void MainWindow::OnMenuAutoFitTree(wxCommandEvent &oEvent)
{
	toggleAutoFit();
}

void MainWindow::toggleMenubar()
{
	if (GetMenuBar() != NULL)
	{
		SetMenuBar(NULL);

		oNavMenu->Check(mnuShowMenubar->GetId(), false);
	}
	else
	{
		SetMenuBar(oMenuBar);

		oNavMenu->Check(mnuShowMenubar->GetId(), true);
	}
}

void MainWindow::toggleToolbar()
{
	if (bToolbarVisible)
	{
		btnUp->Show(false);
		btnHome->Show(false);
		btnFit->Show(false);

		oNavMenu->Check(mnuShowToolbar->GetId(), false);
		
		oMainSizer->Remove(oToolbarSizer);
		
		bToolbarVisible = false;
	}
	else
	{
		oToolbarSizer = new wxBoxSizer(wxHORIZONTAL);
		oToolbarSizer->Add(btnUp, 0, wxALL, 2);
		oToolbarSizer->Add(btnHome, 0, wxALL, 2);
		oToolbarSizer->Add(btnFit, 0, wxALL, 2);
		
		btnUp->Show(true);
		btnHome->Show(true);
		btnFit->Show(true);
		
		oNavMenu->Check(mnuShowToolbar->GetId(), true);
		
		oMainSizer->Prepend(oToolbarSizer);
		
		bToolbarVisible = true;
	}
	oMainSizer->Layout();
}

void MainWindow::toggleLocationBar()
{
	if (bLocationVisible)
	{
		txtLocation->Show(false);
		
		oNavMenu->Check(mnuShowLocation->GetId(), false);
		
		oMainSizer->Remove(oLocationSizer);
		
		bLocationVisible = false;
		
		oTree->SetFocus();
	}
	else
	{
		oLocationSizer = new wxBoxSizer(wxHORIZONTAL);
		oLocationSizer->Add(txtLocation, 1, wxGROW);
		
		txtLocation->Show(true);
		
		oNavMenu->Check(mnuShowLocation->GetId(), true);
		
		oMainSizer->Prepend(oLocationSizer, 0, wxGROW | wxALL, 2);
		
		txtLocation->SetFocus();
		txtLocation->SetInsertionPointEnd();
		
		bLocationVisible = true;
	}
	oMainSizer->Layout();
}

void MainWindow::OnMenuHidden(wxCommandEvent &oEvent)
{
	toggleHidden();
}

void MainWindow::toggleAutoFit()
{
	if (bAutoFitTree)
	{
		oSettings->setGeneral("AUTO_FIT_TREE", "0");
		mnuAutoFitTree->Check(false);
		bAutoFitTree = false;
	}
	else
	{
		oSettings->setGeneral("AUTO_FIT_TREE", "1");
		mnuAutoFitTree->Check(true);
		bAutoFitTree = true;
	}
}

void MainWindow::toggleHidden()
{
	if (oTree->bSeeHidden)
	{
		oNavMenu->Check(mnuShowHidden->GetId(), false);
		oTree->bSeeHidden = false;
	}
	else
	{
		oNavMenu->Check(mnuShowHidden->GetId(), true);
		oTree->bSeeHidden = true;
	}
	
	oTree->SetRoot(oTree->GetItemText(oTree->GetRootItem()), true);
}

void MainWindow::OnMenuSettings(wxCommandEvent &oEvent)
{
	oSettingsWindow->Show(true);
}

void MainWindow::OnLocationEnter(wxCommandEvent &oEvent)
{
	if (UFileHandler::isFile(std::string((char *)txtLocation->GetValue().c_str())))
	{
		oTree->Activate(txtLocation->GetValue());
	}
	oTree->SetFocus();
	toggleLocationBar();
}

void MainWindow::OnMenuExit(wxCommandEvent &oEvent)
{
	Close();
}

void MainWindow::OnMenuOpenInNew(wxCommandEvent &oEvent)
{
	openInNew();
}

void MainWindow::openInNew()
{
	wxString sPath;
	wxString sRunString;
	wxString sAppCommand;
	
	sAppCommand = wxTheApp->argv[0];
	sPath = oTree->getFullpath(oTree->getTreeSelection());
	sRunString = sAppCommand + _(" ") + sPath + _(" &");
	system((char *)sRunString.c_str());
}

void MainWindow::treeFit()
{
	if (FindFocus() != oTree)
	{
		// I'm not sure we should do this. It might cause problems, so assume it's stupid.
		//oTree->SetFocus();

		return; // This is what was in the stable release of 1.0.
	}
	
	// Finding out how big the screen is.
	int iScreenWidth, iScreenHeight;
	wxDisplaySize(&iScreenWidth, &iScreenHeight);
	
	// Finding out how big the tree is.
	int iTreeWidth, iTreeHeight;
	oTree->GetSize(&iTreeWidth, &iTreeHeight);
	
	// Finding out how big our window is.
	// We'll need this later in order to determine the difference between the window and the tree.
	int iWinWidth, iWinHeight;
	GetSize(&iWinWidth, &iWinHeight);
	
	// Here we just check to see how large any object is in the tree.
	// It can vary between icons, text size and so on.
	wxRect oTreeRect;
	oTree->GetBoundingRect(oTree->GetRootItem(), oTreeRect);
	int iItemHeight = oTreeRect.GetHeight();
	
	// This is how big the tree must be in order to lose the scrollbar.
	int iNewTreeHeight = (iItemHeight * oTree->GetCount()) + 50;
	
	int iDiffHeight = iNewTreeHeight - iTreeHeight;
	
	// This defines how much margin space we want.
	char *sTemp = "SCREEN_MARGIN";
	wxString sScreenMargin((wxChar *)oSettings->getGeneral(sTemp).c_str());
	
	long iScreenMargin;
	if (!sScreenMargin.ToLong(&iScreenMargin))
	{
		iScreenMargin = 100;
	}
	int iMaxTreeHeight = iScreenHeight - GetRect().GetTop() - iScreenMargin;
	
	// Here we take the process box into account.
	if (oProcessBox->IsShown())
	{
		int iProcessBoxWidth, iProcessBoxHeight;
		oProcessBox->GetSize(&iProcessBoxWidth, &iProcessBoxHeight);
		iMaxTreeHeight -= iProcessBoxHeight;
	}
	
	// Here we make sure that the window doesn't get bigger than the screen.
	// We'll rather want a scrollbar than that.
	if (iNewTreeHeight > iMaxTreeHeight)
	{
		iNewTreeHeight = iMaxTreeHeight;
		iDiffHeight = iNewTreeHeight - iTreeHeight;
	}
	
	if (iDiffHeight != 0)
	{
		oTree->SetSize(iTreeWidth, iNewTreeHeight);
		SetSize(iWinWidth, iWinHeight + iDiffHeight);
	}
}

void MainWindow::OnTimer(wxTimerEvent &oEvent)
{
	oTimer->Stop();
	
	if (bAutoFitTree)
	{
		treeFit();
	}
	//oTree->checkChanges();
	
	oTimer->Start();
}

void MainWindow::OnTreeKeyDown(wxTreeEvent &oEvent)
{
	// We're throwing the key signal into the USettings object
	// because we're preparing for user-designed keys in the future.
	// In MainWindow, they would have to be hardcoded,
	// but in USettings they can be configurable later.
	// Until we get around fully implementing it,
	// the keycodes are hardcoded into USettings,
	// so that we'll only have to change USettings
	// and not MainWindow as well, when the time comes.
	std::string sKeySignal = oSettings->getKeySignal(oEvent.GetCode());
	
	if (sKeySignal == "ToggleMenubar")
	{
		toggleMenubar();
	}
	else if (sKeySignal == "EditLabel")
	{
		oTree->EditLabel(oTree->getTreeSelection());
	}
	else if (sKeySignal == "DeleteItem")
	{
		oTree->deleteItem();
	}
	else if (sKeySignal == "AddFile")
	{
		oTree->addFile();
	}
	else if (sKeySignal == "ToggleLocationBar")
	{
		toggleLocationBar();
	}
	else if (sKeySignal == "ToggleHidden")
	{
		toggleHidden();
	}
	else if (sKeySignal == "AddFolder")
	{
		oTree->addFolder();
	}
	else if (sKeySignal == "OpenWith")
	{
		oTree->openWith();
	}
	else if (sKeySignal == "Close")
	{
		Close();
	}
	else if (sKeySignal == "GoUp")
	{
		oTree->GoUp();
	}
	else if (sKeySignal == "PressEnd")
	{
		oTree->pressEnd();
	}
	else if (sKeySignal == "PressHome")
	{
		oTree->pressHome();
	}
	else if (sKeySignal == "LeftPress")
	{
		oTree->leftPress();
	}
	else if (sKeySignal == "SelectPrevItem")
	{
		oTree->selectPrevItem();
	}
	else if (sKeySignal == "RightPress")
	{
		oTree->rightPress();
	}
	else if (sKeySignal == "SelectNextItem")
	{
		oTree->selectNextItem();
	}
	else if (sKeySignal == "Activate")
	{
		oTree->Activate(oTree->getTreeSelection());
	}
	else if (sKeySignal == "OpenInNew")
	{
		openInNew();
	}
	else if (sKeySignal == "TreeFit")
	{
		treeFit();
	}
	else if (sKeySignal == "GoHome")
	{
		goHome();
	}
	else if (sKeySignal == "ToggleToolbar")
	{
		toggleToolbar();
	}
}

void MainWindow::OnRightClick(wxTreeEvent &oEvent)
{
	oTree->selectTreeItem(oEvent.GetItem());
	PopupMenu(oActionsMenu, oEvent.GetPoint());
}

void MainWindow::OnMenuOpenWith(wxCommandEvent &oEvent)
{
	oTree->openWith();
}

void MainWindow::OnMenuOpenAndView(wxCommandEvent &oEvent)
{
	wxString sFullpath;
	sFullpath = oTree->getFullpath(oTree->getTreeSelection());
	if (!UFileHandler::isExecutable((char *)sFullpath.c_str()))
	{
		return;
	}
	
	oProcessBox->setCommand(sFullpath);
	
	if (!oProcessBox->IsShown())
	{
		oProcessBox->Show(true);
		btnProcessClose->Show(true);
		oMainSizer->Add(oProcessBox, 0, wxGROW);
		oMainSizer->Add(btnProcessClose, 0, wxALIGN_CENTER, wxALIGN_CENTER);
		oMainSizer->SetItemMinSize(oProcessBox, 10, 120);
		oMainSizer->Layout();
		btnProcessClose->SetFocus();
	}
	
	oProcessBox->run();
}

void MainWindow::OnButtonProcessClose(wxCommandEvent &oEvent)
{
	btnProcessClose->Show(false);
	oProcessBox->Show(false);
	oMainSizer->Remove(btnProcessClose);
	oMainSizer->Remove(oProcessBox);
	oMainSizer->Layout();
	treeFit();
	oTree->SetFocus();
}

void MainWindow::OnHome(wxCommandEvent &oEvent)
{
	goHome();
}

void MainWindow::goHome()
{
	oTree->SetRoot(wxGetUserHome(), false);
	oTree->SetFocus();
}

void MainWindow::OnFit(wxCommandEvent &oEvent)
{
	oTree->SetFocus();
	treeFit();
}


