#ifndef MAINWINDOW_H
#define MAINWINDOW_H
#include <wx/wx.h>

#include <vector>

#include "processbox.h"
#include "filetree.h"
#include "settings.h"
#include "settingswindow.h"

class UFiletree;

class MainWindow : public wxFrame
{
public:
	MainWindow(const wxString &sTitle);

	// Controls.
	UFiletree *oTree;
	
	wxBoxSizer *oMainSizer;
	UProcessBox *oProcessBox;
	wxButton *btnProcessClose;
	
private:
	// Others windows.
	SettingsWindow *oSettingsWindow;
	
	// Functions.
	void toggleMenubar();
	void toggleLocationBar();
	void toggleToolbar();
	void toggleHidden();
	void toggleAutoFit();
	void openInNew();
	void treeFit();
	void goHome();
	// bool CheckChanges(wxTreeItemId iItemID);
	
	// General variables.
	Settings *oSettings;
	bool bLocationVisible;
	bool bToolbarVisible;
	bool bAutoFitTree;
	wxTimer *oTimer;
	wxImageList *oImageList;
	int iRestHeight;
	
	// Sizers.
	wxBoxSizer *oToolbarSizer;
	wxBoxSizer *oLocationSizer;
	wxBoxSizer *oProcessSizer;
	
	// Controls.
	wxTextCtrl *txtLocation;
	wxMenu *oActionsMenu;
	wxMenuBar *oMenuBar;
	wxMenu *oNavMenu;
	wxMenuItem *mnuShowHidden;
	wxMenuItem *mnuShowMenubar;
	wxMenuItem *mnuShowLocation;
	wxMenuItem *mnuShowToolbar;
	wxMenuItem *mnuAutoFitTree;
	wxPanel *oPanel;
	wxBitmapButton *btnUp;
	wxBitmapButton *btnHome;
	wxBitmapButton *btnFit;
	
	// Images & Icons.
	wxBitmap *imgFile;
	
	// Event handlers.
	void OnLocationChange(wxCommandEvent &oEvent);
	void OnLocationEnter(wxCommandEvent &oEvent);
	void OnRunEnter(wxCommandEvent &oEvent);
	void OnUp(wxCommandEvent &oEvent);
	void OnMenuRename(wxCommandEvent &oEvent);
	void OnMenuDelete(wxCommandEvent &oEvent);
	void OnMenuNewFolder(wxCommandEvent &oEvent);
	void OnMenuNewFile(wxCommandEvent &oEvent);
	void OnMenuOpenAndView(wxCommandEvent &oEvent);
	void OnMenuOpenInNew(wxCommandEvent &oEvent);
	void OnMenuOpenWith(wxCommandEvent &oEvent);
	void OnMenuExit(wxCommandEvent &oEvent);
	void OnMenuShowMenubar(wxCommandEvent &oEvent);
	void OnMenuShowLocation(wxCommandEvent &oEvent);
	void OnMenuShowToolbar(wxCommandEvent &oEvent);
	void OnMenuAutoFitTree(wxCommandEvent &oEvent);
	void OnMenuHidden(wxCommandEvent &oEvent);
	void OnMenuSettings(wxCommandEvent &oEvent);
	void OnMenuTreeFit(wxCommandEvent &oEvent);
	void OnTreeKeyDown(wxTreeEvent &oEvent);
	void OnRightClick(wxTreeEvent &oEvent);
	void OnTimer(wxTimerEvent &oEvent);
	void OnButtonProcessClose(wxCommandEvent &oEvent);
	void OnHome(wxCommandEvent &oEvent);
	void OnFit(wxCommandEvent &oEvent);
	
	DECLARE_EVENT_TABLE()
};

#endif

