#include "settings.h"
#include "filehandler.h"

Settings::Settings()
{
	sConfigPath = std::string(getenv("HOME")) + "/.jaffm";
	if (!UFileHandler::isDir(sConfigPath))
	{
		if (UFileHandler::isFile(sConfigPath.c_str()))
		{
			if (!UFileHandler::rmFile(sConfigPath))
			{
				return;
			}
		}
		UFileHandler::mkDir(sConfigPath);
	}
	if (!UFileHandler::isFile(sConfigPath + "/general"))
	{
		FILE *oFile;
		oFile = fopen(std::string(sConfigPath + "/general").c_str(), "w");
		fputs("DEFAULT_APP=gvim\n", oFile);
		fputs("SCREEN_MARGIN=100\n", oFile);
		fputs("SHOW_MENUBAR=1\n", oFile);
		fputs("SHOW_LOCATION=0\n", oFile);
		fputs("SHOW_TOOLBAR=0\n", oFile);
		fputs("AUTO_FIT_TREE=0\n", oFile);
		fclose(oFile);
	}
	if (!UFileHandler::isFile(sConfigPath + "/endings"))
	{
		FILE *oFile;
		oFile = fopen(std::string(sConfigPath + "/endings").c_str(), "w");
		fputs(".avi=mplayer,0\n", oFile);
		fputs(".mpg=mplayer,0\n", oFile);
		fputs(".mpeg=mplayer,0\n", oFile);
		fputs(".mp3=beep-media-player,0\n", oFile);
		fputs(".m3u=beep-media-player,0\n", oFile);
		fputs(".ogg=beep-media-player,0\n", oFile);
		fputs(".xls=gnumeric,0\n", oFile);
		fputs(".doc=abiword,0\n", oFile);
		fputs(".abw=abiword,0\n", oFile);
		fputs(".awt=abiword,0\n", oFile);
		fputs(".jpg=gimp,0\n", oFile);
		fputs(".jpeg=gimp,0\n", oFile);
		fputs(".gif=gimp,0\n", oFile);
		fputs(".png=gimp,0\n", oFile);
		fputs(".pdf=gpdf,0\n", oFile);
		fputs(".sh=sh,1\n", oFile);
		fputs("configure=sh,1\n", oFile);
		fputs("Makefile=sh,1\n", oFile);
		fclose(oFile);
	}

	load();
}

bool Settings::addFileEnding(const std::string &sEnding, const std::string &sRun, bool bTerminal)
{
	bool bInArray = false;
	for (unsigned int i = 0; i < sEndingVector.size(); i++)
	{
		if (sEndingVector[i] == sEnding)
		{
			bInArray = true;
		}
	}
	if (!bInArray)
	{
		sEndingVector.push_back(sEnding);
		sRunVector.push_back(sRun);
		bTerminalVector.push_back(bTerminal);
		return true;
	}
	return false;
}

bool Settings::changeFileEnding(const std::string &sEnding, const std::string &sNewEnding, const std::string &sNewRun, bool bNewTerminal)
{
	bool bInArray = false;
	for (unsigned int i = 0; i < sEndingVector.size(); i++)
	{
		if (sEndingVector[i] == sEnding)
		{
			sEndingVector[i] = sNewEnding;
			sRunVector[i] = sNewRun;
			bTerminalVector[i] = bNewTerminal;
			bInArray = true;
		}
	}
	return bInArray;
}

bool Settings::deleteFileEnding(const std::string &sEnding)
{
	bool bInArray = false;
	std::vector<std::string> sNewEndingVector;
	std::vector<std::string> sNewRunVector;
	for (unsigned int i = 0; i < sEndingVector.size(); i++)
	{
		if (sEndingVector[i] != sEnding)
		{
			sNewEndingVector.push_back(sEndingVector[i]);
			sNewRunVector.push_back(sRunVector[i]);
		}
		else
		{
			bInArray = true;
		}
	}
	sEndingVector = sNewEndingVector;
	sRunVector = sNewRunVector;
	return bInArray;
}

std::vector<std::string> Settings::getFileEndings()
{
	return sEndingVector;
}

std::string Settings::getRunString(const std::string &sEnding)
{
	std::string sResult;
	for (unsigned int i = 0; i < sEndingVector.size(); i++)
	{
		if (sEndingVector[i] == sEnding)
		{
			// sResult = std::string::Format(sRunVector[i].c_str(), sEndingVector[i].c_str());
			// Aren't we supposed to format this? :/
			sResult = sRunVector[i];
		}
	}
	return sResult;
}

int Settings::count()
{
	return sEndingVector.size();
}

void Settings::load()
{
	std::string sLine;
	int iLoc;
	int iNextComma;
	std::string sApp;
	std::string sRest;

	std::vector<std::string> sNewEndingVector;
	std::vector<std::string> sNewRunVector;
	std::vector<bool> bNewTerminalVector;
	
	sEndingVector = sNewEndingVector;
	sRunVector = sNewRunVector;
	bTerminalVector = bNewTerminalVector;
	
	std::vector<std::string> oEndings = UFileHandler::importFile(sConfigPath + "/endings");
	for (unsigned int i = 0; i < oEndings.size(); i++)
	{
		sLine = oEndings[i];
		iLoc = sLine.find('=');
		iNextComma = sLine.find(',');
		sApp = (iNextComma == -1 ? sLine.substr(iLoc + 1) : sLine.substr(iLoc + 1, iNextComma - iLoc - 1));
		sRest = sLine.substr(iNextComma + 1);
		addFileEnding(sLine.substr(0, iLoc), sApp, (sRest.substr(0, 1) == "1"));
	}
	
	std::vector<std::string> oGeneral = UFileHandler::importFile(sConfigPath + "/general");
	for (unsigned int i = 0; i < oGeneral.size(); i++)
	{
		sLine = oGeneral[i];
		iLoc = sLine.find('=');
		setGeneral(sLine.substr(0, iLoc).c_str(), sLine.substr(iLoc + 1).c_str());
	}
}

void Settings::save()
{
	FILE *oFile;
	oFile = fopen(std::string(sConfigPath + "/general").c_str(), "w+");
	for (unsigned int i = 0; i < sGeneralKeys.size(); i++)
	{
		fputs(std::string(sGeneralKeys[i] + "=" + sGeneralValues[i] + "\n").c_str(), oFile);
	}
	fclose(oFile);
	
	oFile = fopen(std::string(sConfigPath + "/endings").c_str(), "w+");
	for (unsigned int i = 0; i < sEndingVector.size(); i++)
	{
		fputs(std::string(sEndingVector[i] + "=" + sRunVector[i] + "," + (bTerminalVector[i] ? "1" : "0") + "\n").c_str(), oFile);
	}
	fclose(oFile);
}

std::string Settings::getEnding(const std::string &sItem)
{
	std::string sFilename;
	sFilename = sItem.substr(sItem.rfind('/') + 1);
	int iLoc = sFilename.rfind('.');
	if (iLoc == -1)
	{
		return sFilename;
	}
	else
	{
		std::string sEnding = sFilename.substr(iLoc);
		return sEnding;
	}
}

bool Settings::exists(const std::string &sEnding)
{
	bool bExists = false;
	for (unsigned int i = 0; i < sEndingVector.size(); i++)
	{
		if (sEndingVector[i] == sEnding)
		{
			bExists = true;
		}
	}
	return bExists;
}

void Settings::setGeneral(const std::string &sVarname, const std::string &sValue)
{
	bool bExists = false;
	for (unsigned int i = 0; i < sGeneralKeys.size(); i++)
	{
		if (sGeneralKeys[i] == sVarname)
		{
			bExists = true;
			sGeneralValues[i] = sValue;
		}
	}
	if (!bExists)
	{
		sGeneralKeys.push_back(sVarname);
		sGeneralValues.push_back(sValue);
	}
}

std::string Settings::getGeneral(const std::string &sVarname)
{
	std::string sReturn;
	for (unsigned int i = 0; i < sGeneralKeys.size(); i++)
	{
		if (sGeneralKeys[i] == sVarname)
		{
			sReturn = sGeneralValues[i];
		}
	}
	return sReturn;
}

std::string Settings::getKeySignal(int iCode)
{
	std::string sReturn;
	
	printf("Key: %c, %d\n", iCode, iCode);
	
	if (iCode == 'm')
	{
		sReturn = "ToggleMenubar";
	}
	else if (iCode == 'r')
	{
		sReturn = "EditLabel";
	}
	else if (iCode == 127) // Delete
	{
		sReturn = "DeleteItem";
	}
	else if (iCode == 'f')
	{
		sReturn = "AddFile";
	}
	else if (iCode == 'l')
	{
		sReturn = "ToggleLocationBar";
	}
	else if (iCode == 'h')
	{
		sReturn = "ToggleHidden";
	}
	else if (iCode == 'd')
	{
		sReturn = "AddFolder";
	}
	else if (iCode == 111)
	{
		sReturn = "OpenWith";
	}
	else if (iCode == 27) // Escape
	{
		sReturn = "Close";
	}
	else if (iCode == 8) // Backspace
	{
		sReturn = "GoUp";
	}
	else if (iCode == 314) // End
	{
		sReturn = "PressEnd";
	}
	else if (iCode == 315) // Home
	{
		sReturn = "PressHome";
	}
	else if (iCode == 316) // Left
	{
		sReturn = "LeftPress";
	}
	else if (iCode == 317) // Up
	{
		sReturn = "SelectPrevItem";
	}
	else if (iCode == 318) // Right
	{
		sReturn = "RightPress";
	}
	else if (iCode == 319) // Down
	{
		sReturn = "SelectNextItem";
	}
	else if (iCode == 13) // Enter
	{
		sReturn = "Activate";
	}
	else if (iCode == 119)
	{
		sReturn = "OpenInNew";
	}
	else if (iCode == 105)
	{
		sReturn = "TreeFit";
	}
	else if (iCode == 'u')
	{
		sReturn = "GoHome";
	}
	else if (iCode == 't')
	{
		sReturn = "ToggleToolbar";
	}

	return sReturn;
}

bool Settings::getTerminal(const std::string &sEnding)
{
	bool bReturn = false;
	for (unsigned int i = 0; i < sEndingVector.size(); i++)
	{
		if (sEndingVector[i] == sEnding && bTerminalVector[i])
		{
			bReturn = true;
			break;
		}
	}
	return bReturn;
}

